@@ -32,6 +32,7 @@
 
 #include "server/camel-ews-settings.h"
 #include "server/e-ews-connection.h"
+#include "server/e-ews-connection-utils.h"
 #include "server/e-ews-oof-settings.h"
 
 #define E_MAIL_CONFIG_EWS_OOO_PAGE_GET_PRIVATE(obj) \
@@ -422,8 +423,7 @@ mail_config_ews_ooo_page_constructed (GObject *object)
 	page = E_MAIL_CONFIG_EWS_OOO_PAGE (object);
 
 	/* Chain up to parent's constructed() method. */
-	G_OBJECT_CLASS (e_mail_config_ews_ooo_page_parent_class)->
-		constructed (object);
+	G_OBJECT_CLASS (e_mail_config_ews_ooo_page_parent_class)->constructed (object);
 
 	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
@@ -448,6 +448,7 @@ mail_config_ews_ooo_page_constructed (GObject *object)
 	text = _("The messages specified below will be automatically sent to "
 		 "each internal and external person who sends a mail to you.");
 	widget = gtk_label_new (text);
+	gtk_label_set_max_width_chars (GTK_LABEL (widget), 80);
 	gtk_label_set_line_wrap (GTK_LABEL (widget), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, FALSE, 0);
@@ -744,6 +745,9 @@ mail_config_ews_ooo_page_submit (EMailConfigPage *page,
 	e_ews_oof_settings_set_internal_reply (priv->oof_settings, text);
 	g_free (text);
 
+	e_ews_oof_settings_set_external_audience (priv->oof_settings,
+		gtk_combo_box_get_active (GTK_COMBO_BOX (priv->external_audience)));
+
 	text_buffer = priv->external_reply;
 	gtk_text_buffer_get_bounds (text_buffer, &start, &end);
 	text = gtk_text_buffer_get_text (text_buffer, &start, &end, FALSE);
@@ -790,6 +794,20 @@ mail_config_ews_ooo_page_submit_finish (EMailConfigPage *page,
 	return !g_simple_async_result_propagate_error (simple, error);
 }
 
+static gboolean
+mail_config_ews_ooo_page_get_without_password (ESourceAuthenticator *auth)
+{
+	EMailConfigEwsOooPage *page;
+	CamelSettings *settings;
+	CamelEwsSettings *ews_settings;
+
+	page = E_MAIL_CONFIG_EWS_OOO_PAGE (auth);
+	settings = mail_config_ews_ooo_page_get_settings (page);
+	ews_settings = CAMEL_EWS_SETTINGS (settings);
+
+	return e_ews_connection_utils_get_without_password (ews_settings);
+}
+
 static ESourceAuthenticationResult
 mail_config_ews_ooo_page_try_password_sync (ESourceAuthenticator *auth,
                                             const GString *password,
@@ -922,6 +940,8 @@ e_mail_config_ews_ooo_page_interface_init (EMailConfigPageInterface *iface)
 static void
 e_mail_config_ews_ooo_page_authenticator_init (ESourceAuthenticatorInterface *iface)
 {
+	iface->get_without_password =
+		mail_config_ews_ooo_page_get_without_password;
 	iface->try_password_sync =
 		mail_config_ews_ooo_page_try_password_sync;
 }
