@@ -31,6 +31,7 @@
 struct _CamelEwsSettingsPrivate {
 	GMutex property_lock;
 	gboolean check_all;
+	gboolean listen_notifications;
 	gboolean filter_junk;
 	gboolean filter_junk_inbox;
 	gboolean oab_offline;
@@ -49,6 +50,7 @@ enum {
 	PROP_0,
 	PROP_AUTH_MECHANISM,
 	PROP_CHECK_ALL,
+	PROP_LISTEN_NOTIFICATIONS,
 	PROP_EMAIL,
 	PROP_FILTER_JUNK,
 	PROP_FILTER_JUNK_INBOX,
@@ -74,6 +76,35 @@ G_DEFINE_TYPE_WITH_CODE (
 	G_IMPLEMENT_INTERFACE (
 		CAMEL_TYPE_NETWORK_SETTINGS, NULL))
 
+static gboolean
+ews_settings_transform_host_url_to_host_cb (GBinding *binding,
+					    const GValue *host_url_value,
+					    GValue *host_value,
+					    gpointer user_data)
+{
+	const gchar *host_url;
+
+	host_url = g_value_get_string (host_url_value);
+	if (host_url && *host_url) {
+		SoupURI *uri;
+
+		uri = soup_uri_new (host_url);
+		if (uri) {
+			const gchar *host;
+
+			host = soup_uri_get_host (uri);
+			if (!host || !*host)
+				host = "";
+
+			g_value_set_string (host_value, host);
+
+			soup_uri_free (uri);
+		}
+	}
+
+	return TRUE;
+}
+
 static void
 ews_settings_set_property (GObject *object,
                             guint property_id,
@@ -93,6 +124,12 @@ ews_settings_set_property (GObject *object,
 				g_value_get_boolean (value));
 			return;
 
+		case PROP_LISTEN_NOTIFICATIONS:
+			camel_ews_settings_set_listen_notifications (
+				CAMEL_EWS_SETTINGS (object),
+				g_value_get_boolean (value));
+			return;
+
 		case PROP_EMAIL:
 			camel_ews_settings_set_email (
 				CAMEL_EWS_SETTINGS (object),
@@ -214,6 +251,13 @@ ews_settings_get_property (GObject *object,
 				CAMEL_EWS_SETTINGS (object)));
 			return;
 
+		case PROP_LISTEN_NOTIFICATIONS:
+			g_value_set_boolean (
+				value,
+				camel_ews_settings_get_listen_notifications (
+				CAMEL_EWS_SETTINGS (object)));
+			return;
+
 		case PROP_EMAIL:
 			g_value_take_string (
 				value,
@@ -380,6 +424,18 @@ camel_ews_settings_class_init (CamelEwsSettingsClass *class)
 			G_PARAM_CONSTRUCT |
 			G_PARAM_STATIC_STRINGS));
 
+	g_object_class_install_property (
+		object_class,
+		PROP_LISTEN_NOTIFICATIONS,
+		g_param_spec_boolean (
+			"listen-notifications",
+			"Listen Notifications",
+			"Whether to listen for server notification",
+			FALSE,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT |
+			G_PARAM_STATIC_STRINGS));
+
 	g_object_class_install_property (
 		object_class,
 		PROP_EMAIL,
@@ -554,6 +610,51 @@ camel_ews_settings_init (CamelEwsSettings *settings)
 {
 	settings->priv = CAMEL_EWS_SETTINGS_GET_PRIVATE (settings);
 	g_mutex_init (&settings->priv->property_lock);
+
+	g_object_bind_property_full (settings, "hosturl",
+				     settings, "host",
+				     G_BINDING_DEFAULT,
+				     ews_settings_transform_host_url_to_host_cb,
+				     NULL,
+				     NULL,
+				     NULL);
+}
+
+/**
+ * camel_ews_settings_get_auth_mechanism:
+ * @settings: a #CamelEwsSettings
+ *
+ * Returns a #EwsAuthType enum value indicating which authentication
+ * method to use.
+ *
+ * Returns: authentication method to use for this account
+ *
+ * Since: 3.14
+ **/
+EwsAuthType
+camel_ews_settings_get_auth_mechanism (CamelEwsSettings *settings)
+{
+	EwsAuthType result;
+	gchar *auth_mech = NULL;
+
+	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), FALSE);
+
+	g_object_get (G_OBJECT (settings), "auth-mechanism", &auth_mech, NULL);
+
+	/* The value for "NTLM" is a special case. Sometimes it's NULL,
+	 * and sometimes it's "". But never "NTLM". No, that would be too
+	 * simple. (I think it's for backward-compatibility with old
+	 * profiles, so they default to NTLM). */
+	if (auth_mech && g_ascii_strcasecmp (auth_mech, "PLAIN") == 0)
+		result = EWS_AUTH_TYPE_BASIC;
+	else if (auth_mech && g_ascii_strcasecmp (auth_mech, "GSSAPI") == 0)
+		result = EWS_AUTH_TYPE_GSSAPI;
+	else
+		result = EWS_AUTH_TYPE_NTLM;
+
+	g_free (auth_mech);
+
+	return result;
 }
 
 /**
@@ -597,6 +698,47 @@ camel_ews_settings_set_check_all (CamelEwsSettings *settings,
 	g_object_notify (G_OBJECT (settings), "check-all");
 }
 
+/**
+ * camel_ews_settings_get_listen_notifications:
+ * @settings: a #CamelEwsSettings
+ *
+ * Returns whether to listen for server notifications.
+ *
+ * Returns: whether to listen for server notifications
+ *
+ * Since: 3.12
+ **/
+gboolean
+camel_ews_settings_get_listen_notifications (CamelEwsSettings *settings)
+{
+	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), FALSE);
+
+	return settings->priv->listen_notifications;
+}
+
+/**
+ * camel_ews_settings_set_listen_notifications:
+ * @settings: a #CamelEwsSettings
+ * @listen_notifications: whether to listen for server notifications
+ *
+ * Sets whether to listen for server notifications.
+ *
+ * Since: 3.12
+ **/
+void
+camel_ews_settings_set_listen_notifications (CamelEwsSettings *settings,
+					     gboolean listen_notifications)
+{
+	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
+
+	if ((settings->priv->listen_notifications ? 1 : 0) == (listen_notifications ? 1 : 0))
+		return;
+
+	settings->priv->listen_notifications = listen_notifications;
+
+	g_object_notify (G_OBJECT (settings), "listen-notifications");
+}
+
 /**
  * camel_ews_settings_get_folders_initialized:
  * @settings: a #CamelEwsSettings
